@@ -2,19 +2,20 @@
 from moviepy.editor import *
 from moviepy.video.fx import loop, mask_color, crop
 from bgeditor.common import utils
-from bgeditor.common.utils import cache_file, download_file, upload_file
+from bgeditor.common.utils import cache_file, download_file, upload_file, upload_static_file
 from bgeditor.dao.Lyric import Lyric
 from bgeditor.dao.Matrix import Matrix
 from bgeditor.dao import MusicHelper
 from PIL import Image
 import numpy as np
 import requests, time
-from bgeditor.dao import FFmpeg
-from bgeditor.dao.FFmpeg import create_suource_can_loop_path, create_loop, create_loop_audio, create_video_audio, merge_intro_outro
+from bgeditor.dao import FFmpeg, LyricStatic
+from bgeditor.dao.FFmpeg import create_suource_can_loop_path, create_loop, create_loop_audio, create_video_audio, merge_intro_outro, merge_list_video
 from proglog import ProgressBarLogger
 from bgeditor.common.utils import get_dir
 import uuid, textwrap
 import math
+import json
 from moviepy.video.fx import make_loopable
 class MyBarLogger(ProgressBarLogger):
     def __init__(self,job_id, mf_server):
@@ -114,6 +115,8 @@ def create_video(list_comp_data, path_video, job_id, mf_server):
     outroPath = None
     firstImageOrVideoBG=[] #purpose for loop lyric outro
     indexVidBG=0
+    arr_static_lyric_id=[]
+    is_mix_lyric=False
     #check if videos are compilation
     for comp in arr_comps:
         if indexVidBG < 5 and  (comp.type == "image"):
@@ -130,6 +133,9 @@ def create_video(list_comp_data, path_video, job_id, mf_server):
                 if is_outro_claim:
                     outro_claim_topic =  comp.outro_claim_topic
 
+        if comp.type == "mix_lyric":
+            arr_static_lyric_id = LyricStatic.create_lyric_static(firstImageOrVideoBG, comp.mix_data, comp.font_url, int(comp.lyric_lines))
+            is_mix_lyric = True
         if comp.type == "compilation":
             is_compilation = True
             #test
@@ -138,11 +144,11 @@ def create_video(list_comp_data, path_video, job_id, mf_server):
     for comp in arr_comps:
         if comp.type == "compilation":
             continue
-        if comp.type=="video":
+        if comp.type == "video":
             cnt_effect_vid += 1
-            is_cont=False
+            is_cont = False
             if comp.is_intro:
-                introPath = os.path.join(get_dir('coolbg_ffmpeg'),str(uuid.uuid4()) + "-intro.mp4")
+                introPath = os.path.join(get_dir('coolbg_ffmpeg'), str(uuid.uuid4()) + "-intro.mp4")
                 rs = comp.get_clip()
                 if not rs.audio:
                     rs.close()
@@ -189,14 +195,18 @@ def create_video(list_comp_data, path_video, job_id, mf_server):
             if tmp_start <0:
                 tmp_start=0
             item.set_start(tmp_start)
+
+    #----------------------Render---------------------------#
     logger = MyBarLogger(job_id, mf_server)
     if not is_compilation:
+        #make Lyric
         if max_duration <= 0:
             max_duration = max_real_duration
         final_clip = CompositeVideoClip(arr_composite).subclip(0, max_duration)
         final_clip.write_videofile(path_video, bitrate='4M', fps=24, codec='libx264', logger=logger)
         final_clip.close()
     else:
+        # make Compilation
         if audio_compilation_path is None:
             raise Exception("Audio Compilation Error!!!")
         cnt_real_duration = 1
@@ -245,6 +255,11 @@ def create_video(list_comp_data, path_video, job_id, mf_server):
     final_vid = VideoFileClip(path_video)
     duration_f = final_vid.duration
     final_vid.close()
+    if is_mix_lyric:
+        arr_vids = []
+        arr_vids.append(path_video)
+        arr_vids += LyricStatic.load_lyric_video(arr_static_lyric_id)
+        path_video = merge_list_video(arr_vids, True)
     if is_lyric_outro:
         lyricOutroPath = create_lyric_outro(deezer_artist_id, lyric_outro_repeat, firstImageOrVideoBG, outro_claim_topic)
         path_video = merge_intro_outro(path_video, None, lyricOutroPath)
@@ -297,6 +312,8 @@ class Component:
             return ImageComp(json_data)
         if json_data['type'] == "video":
             return VideoComp(json_data)
+        if json_data['type'] == "mix_lyric":
+            return MixLyricComp(json_data)
         if json_data['type'] == "lyric":
             return LyricComp(json_data)
         if json_data['type'] == "element":
@@ -568,6 +585,35 @@ class VideoComp(Component):
             rs = rs.resize((self.position['width'], self.position['height']))
         return rs
 
+class MixLyricComp(Component):
+    def __init__(self, json_data):
+        super().__init__(json_data)
+        self.font_url = json_data['font_url']
+        self.font_size = json_data['fontSize']
+        self.bg_color = json_data['backgroundColor']
+        if self.bg_color is None or self.bg_color == "":
+            self.bg_color="transparent"
+        self.color = json_data['color']
+        self.stroke_color = json_data['stroke_color']
+        self.stroke_width = json_data['stroke_width']
+        self.audio_url = json_data['audio_url']
+        self.audio_ext = json_data['audio_ext']
+        self.lyric_sync = json_data['lyric_sync']
+        self.wrap_width = json_data['wrap_width']
+        self.lyric_moving = json_data['lyric_moving']
+        self.fade_in = json_data['fade_in']
+        self.fade_out = json_data['fade_out']
+        self.mix_data = json_data['mix_data']
+        self.lyric_lines = json_data['lyric_lines']
+
+    def get_clip(self):
+        self.lyric = Lyric(self.lyric_sync, self.font_url, self.font_size, self.color,
+                           self.audio_moviepy.duration, self.stroke_color, self.stroke_width, self.bg_color,
+                           self.lyric_moving, self.fade_in, self.fade_out, self.wrap_width, self.position['width'], self.position['height'])
+        self.duration = self.audio_moviepy.duration
+        self.lyric.init()
+        self.lyric.optimize_font()
+        return self.lyric.make()
 
 class LyricComp(Component):
     def __init__(self, json_data):
